#version 120

//***************************ADJUSTABLE VARIABLES***************************//
//***************************ADJUSTABLE VARIABLES***************************//
//***************************ADJUSTABLE VARIABLES***************************//

//***************************TORCH***************************//
	vec3 torchcolor = vec3(1.0,0.32,0.1);		//RGB - Red, Green, Blue /
	#define TORCH_ATTEN 5.0						//how much the torch light will be attenuated (decrease if you want that the torches cover a bigger area))
	#define TORCH_INTENSITY 2.6					//torch light intensity 
	
//***************************GODRAYS***************************//

#define GODRAYS
	const float exposure = 1;		
	const float density = 1;			
	const int NUM_SAMPLES = 15;			//increase this for better quality at the cost of performance
	const float grnoise = 0.0;		//amount of noise /0.0 is default
#define	GODRAYS_SMOOTHNESS 1.0
	
//***************************REFLECTIONS***************************//	

#define WATER_REFLECTIONS			
	#define REFLECTION_STRENGTH 2.0
	
#define RAIN_REFLECTIONS

#define SPECULAR_REFLECTIONS

//***************************VISUALS***************************//
	
//#define MOTIONBLUR
	#define MOTIONBLUR_AMOUNT 1.0
	//#define HQ_MOTIONBLUR			//High quality motionblur only enable one!
	#define LQ_MOTIONBLUR			//Low quality motionblur only enable one!

//***************************END OF ADJUSTABLE VARIABLES***************************//
//***************************END OF ADJUSTABLE VARIABLES***************************//
//***************************END OF ADJUSTABLE VARIABLES***************************//



//don't touch these lines if you don't know what you do!
const int maxf = 3;				//number of refinements
const float stp = 1.0;			//size of one step for raytracing algorithm
const float ref = 0.05;			//refinement multiplier
const float inc = 2.2;			//increasement factor at each step

const int Gmaxf = 3;				//number of refinements
const float Gstp = 1.0;			//size of one step for raytracing algorithm
const float Gref = 0.05;			//refinement multiplier
const float Ginc = 2.2;			//increasement factor at each step

varying vec4 texcoord;
varying vec3 sunlight;
varying vec3 moonlight;
varying vec3 lightVector;
varying vec3 ambient_color;
varying float SdotU;
varying float MdotU;
varying vec3 sunVec;
varying vec3 moonVec;
varying vec3 upVec;
varying float sunVisibility;
varying float moonVisibility;

varying float eyeAdapt;

uniform sampler2D composite;
uniform sampler2D gaux4;
uniform sampler2D gaux1;
uniform sampler2D gaux3;
uniform sampler2D gaux2;
uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D depthtex2;
uniform sampler2D gnormal;
uniform sampler2D gdepth;
uniform sampler2D gdepthtex;
uniform sampler2D noisetex;

uniform vec3 sunPosition;
uniform vec3 moonPosition;
uniform vec3 upPosition;
uniform vec3 cameraPosition;
uniform vec3 skyColor;

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferPreviousProjection;
uniform mat4 gbufferPreviousModelView;
uniform vec3 previousCameraPosition;
uniform ivec2 eyeBrightness;
uniform ivec2 eyeBrightnessSmooth;
uniform int isEyeInWater;
uniform int worldTime;
uniform float far;
uniform float near;
uniform float aspectRatio;
uniform float viewWidth;
uniform float viewHeight;
uniform float rainStrength;
uniform float wetness;
uniform float frameTimeCounter;
uniform int fogMode;

float timefract = worldTime;

//Raining
float rainx = clamp(rainStrength, 0.0f, 1.0f)/1.0f;
float wetx  = clamp(wetness, 0.0f, 1.0f);

//Calculate Time of Day
float TimeSunrise  = ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0) + (1.0 - (clamp(timefract, 0.0, 4000.0)/4000.0));
float TimeNoon     = ((clamp(timefract, 0.0, 4000.0)) / 4000.0) - ((clamp(timefract, 8000.0, 12000.0) - 8000.0) / 4000.0);
float TimeSunset   = ((clamp(timefract, 8000.0, 12000.0) - 8000.0) / 4000.0) - ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0);
float TimeMidnight = ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0) - ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0);

float pw = 1.0/ viewWidth;
float ph = 1.0/ viewHeight;

	float matflag = texture2D(gaux1,texcoord.xy).g;
	
	vec3 fogclr = mix(gl_Fog.color.rgb,vec3(0.2,0.2,0.2),rainStrength)*ambient_color;
	vec3 fogclr2 = fogclr;
	vec3 fogclr3 = fogclr;
	vec3 fogclr4 = fogclr;
	
    vec3 fragpos = vec3(texcoord.st, texture2D(gdepthtex, texcoord.st).r);
    vec3 normal = texture2D(gnormal, texcoord.st).rgb * 2.0 - 1.0;
	
	float sky_lightmap = texture2D(gaux1,texcoord.xy).r;
	
	float iswet = wetness*pow(sky_lightmap,5.0)*sqrt(0.5+max(dot(normal,normalize(upPosition)),0.0));
	
	int iswater = int(matflag > 0.04 && matflag < 0.07);

	vec3 specular = pow(texture2D(gaux3,texcoord.xy).rgb,vec3(2.2));
	float specmap = (specular.r+specular.g*(iswet));
    vec4 color = texture2D(composite,texcoord.xy);

	
	const float speed = 1.5;
	float light_jitter = 1.0-sin(frameTimeCounter*1.4*speed+cos(frameTimeCounter*1.9*speed))*0.025;			//little light variations
	float torch_lightmap = pow(texture2D(gaux1,texcoord.xy).b*light_jitter,TORCH_ATTEN)*TORCH_INTENSITY;


float ld(float depth) {
    return (2.0 * near) / (far + near - depth * (far - near));
}

vec3 nvec3(vec4 pos) {
    return pos.xyz/pos.w;
}

vec4 nvec4(vec3 pos) {
    return vec4(pos.xyz, 1.0);
}

float cdist(vec2 coord) {
	return max(abs(coord.x-0.5),abs(coord.y-0.5))*2.0;
}

vec4 raytrace(vec3 fragpos, vec3 normal) {
    vec4 color = vec4(0.0);
    vec3 start = fragpos;
    vec3 rvector = normalize(reflect(normalize(fragpos), normalize(normal)));
    vec3 vector = stp * rvector;
    vec3 oldpos = fragpos;
    fragpos += vector;
	vec3 tvector = vector;
    int sr = 0;
	

	    for(int i=0;i<30;i++){
        vec3 pos = nvec3(gbufferProjection * nvec4(fragpos)) * 0.5 + 0.5;
        if(pos.x < 0 || pos.x > 1 || pos.y < 0 || pos.y > 1 || pos.z < 0 || pos.z > 1.0) break;
        vec3 spos = vec3(pos.st, texture2D(depthtex1, pos.st).r);
        spos = nvec3(gbufferProjectionInverse * nvec4(spos * 2.0 - 1.0));
        float err = distance(fragpos.xyz,spos.xyz);
        if(err < pow(length(vector)*pow(length(tvector),0.11),1.1)*1.1){

                sr++;
                if(sr >= maxf){
                    float border = clamp(1.0 - pow(cdist(pos.st), 5.0), 0.0, 1.0);
                    color = texture2D(composite, pos.st);
					color.a = 1.0;
                    color.a *= border;
                    break;
                }
				tvector -=vector;
                vector *=ref;
				
        
}
        vector *= inc;
        oldpos = fragpos;
        tvector += vector;
		fragpos = start + tvector;
    }

    return color;
}

vec4 raytraceGround(vec3 fragpos, vec3 normal) {
if (iswater < 0.9) {
    vec4 color = vec4(0.0);
    vec3 start = fragpos;
    vec3 rvector = normalize(reflect(normalize(fragpos), normalize(normal)));
    vec3 vector = Gstp * rvector;
    vec3 oldpos = fragpos;
    fragpos += vector;
	vec3 tvector = vector;
    int sr = 0;
    for(int i=0;i<30;i++){
        vec3 pos = nvec3(gbufferProjection * nvec4(fragpos)) * 0.5 + 0.5;
		if(pos.x < 0 || pos.x > 1 || pos.y < 0 || pos.y > 1 || pos.z < 0 || pos.z > 1.0) break;
        vec3 spos = vec3(pos.st, texture2D(depthtex1, pos.st).r);
        spos = nvec3(gbufferProjectionInverse * nvec4(spos * 2.0 - 1.0));
        float err = distance(fragpos.xyz,spos.xyz);
        if(err < pow(length(vector)*pow(length(tvector),0.11),1.1)*1.1){

                sr++;
                if(sr >= maxf){
                    float border = clamp(1.0 - pow(cdist(pos.st), 20.0), 0.0, 1.0);
                    color = texture2D(composite, pos.st);
					color.a = 1.0;
                    color.a *= border;
                    break;
                }
				tvector -=vector;
                vector *=Gref;
				
        
}
        vector *= Ginc;
        oldpos = fragpos;
        tvector += vector;
		fragpos = start + tvector;
    }
    return color;
	}
}

float getnoise(vec2 pos) {
return abs(fract(sin(dot(pos ,vec2(18.9898f,28.633f))) * 4378.5453f));

}

float getRainPuddles(vec3 fposition){

	
	vec3 pPos = vec3(texcoord.st, texture2D(depthtex1, texcoord.st).r);
	pPos = nvec3(gbufferProjectionInverse * nvec4(pPos * 2.0 - 1.0));
	vec4 pUw = gbufferModelViewInverse * vec4(pPos,1.0);	
	vec3 worldPos = (pUw.xyz + cameraPosition.xyz);

	vec2 coord = (worldPos.xz/10000);

	float rainPuddles = texture2D(noisetex, fract(coord.xy*8)).x;
	rainPuddles += texture2D(noisetex, fract(coord.xy*4)).x;
	rainPuddles += texture2D(noisetex, fract(coord.xy*2)).x;
	rainPuddles += texture2D(noisetex, fract(coord.xy/2)).x;
	
	float strength = max(rainPuddles-1.9,0.0);
	float dL = 0.5;
	float L = (1.0 - (pow(dL,strength)));
	
	
	return L;
}



//////////////////////////////main//////////////////////////////
//////////////////////////////main//////////////////////////////
//////////////////////////////main//////////////////////////////
//////////////////////////////main//////////////////////////////
//////////////////////////////main//////////////////////////////

void main() {
	int iswater = int(matflag > 0.04 && matflag < 0.07);
	int land = int(matflag < 0.03);
	int hand  = int(matflag > 0.75 && matflag < 0.85);

	vec4 depth  = texture2D(depthtex2, texcoord.st);
	
	vec4 currentPosition = vec4(texcoord.x * 2.0 - 1.0, texcoord.y * 2.0 - 1.0, 2.0 * depth.x - 1.0, 1.0);
	
	vec4 fragposition = gbufferProjectionInverse * currentPosition;
	fragposition = gbufferModelViewInverse * fragposition;
	fragposition /= fragposition.w;
	fragposition.xyz += cameraPosition;
	
	float pixeldepth = texture2D(depthtex0,texcoord.xy).x;
	
	vec4 fragposition2 = gbufferProjectionInverse * vec4(texcoord.s * 2.0f - 1.0f, texcoord.t * 2.0f - 1.0f, 2.0f * pixeldepth - 1.0f, 1.0f);
	fragposition2 /= fragposition2.w;
	
	
	vec4 previousPosition = fragposition;
	previousPosition.xyz -= previousCameraPosition;
	previousPosition = gbufferPreviousModelView * previousPosition;
	previousPosition = gbufferPreviousProjection * previousPosition;
	previousPosition /= previousPosition.w;
	
	vec4 worldposition = vec4(0.0);
	
	worldposition = gbufferModelViewInverse * fragposition2;	
	
#ifdef MOTIONBLUR	
	if (hand < 0.1) {
	#ifdef LQ_MOTIONBLUR
	vec2 velocity = (currentPosition - previousPosition, previousPosition - currentPosition).st * (0.01*MOTIONBLUR_AMOUNT*(1.0-TimeMidnight));
	#endif
	#ifdef HQ_MOTIONBLUR
	vec2 velocity = (currentPosition - previousPosition).st * (0.01*MOTIONBLUR_AMOUNT*(1.0-TimeMidnight));
	#endif

	int samples = 1;

	vec2 coord = texcoord.st + velocity;
	#ifdef HQ_MOTIONBLUR
	for (int i = 0; i < 16; ++i, coord += velocity) {
		if (coord.s > 1.0 || coord.t > 1.0 || coord.s < 0.0 || coord.t < 0.0) {
			break;
		}
            color += texture2D(composite, coord),color.rgb;
			++samples;
	}
	#endif
		#ifdef LQ_MOTIONBLUR
		for (int i = 0; i < 4; ++i, coord += velocity) {
		if (coord.s > 1.0 || coord.t > 1.0 || coord.s < 0.0 || coord.t < 0.0) {
			break;
		}
            color += texture2D(composite, coord),color.rgb;
			++samples;
	}
	#endif

	color = (color/1.0)/samples;
	}
#endif
	vec3 rainfogclr = vec3(0.1,0.1,0.12)*rainx*(1-TimeMidnight*0.9);
	
	vec3 sclr = vec3(75,115,244)/540*REFLECTION_STRENGTH/REFLECTION_STRENGTH*TimeNoon;
	vec3 sclrn = vec3(0.75,1,1.5)/1.75*REFLECTION_STRENGTH/REFLECTION_STRENGTH*TimeMidnight;
	vec3 sclrsr = vec3(20,20,25)/30*REFLECTION_STRENGTH/REFLECTION_STRENGTH*TimeSunrise;
	vec3 sclrss = vec3(20,20,25)/30*REFLECTION_STRENGTH/REFLECTION_STRENGTH*TimeSunset;
	vec3 skycolorreflection = vec3((sclr+sclrn+sclrsr+sclrss)*(1.0-rainx)+(rainfogclr)*(3-TimeMidnight*-6));
	
	fragpos = nvec3(gbufferProjectionInverse * nvec4(fragpos * 2.0 - 1.0));

	float fresnel_pow = mix(pow(1.0-(specular.b+specular.g)/2.0,1.25+iswet*0.75)*3.5,3.0,iswater);
		float normalDotEye = dot(normal, normalize(fragpos));
		float fresnel = clamp(pow(1.0 + normalDotEye, 1.0),0.0,1.0);
		vec4 reflection;
	if (iswater > 0.9) {
	#ifdef WATER_REFLECTIONS
		reflection = raytrace(fragpos, normal);
		reflection.rgb = mix(skycolorreflection*(1.0-TimeMidnight*0.85)*(1.0-TimeSunrise*0.35)*(1.0-TimeSunset*0.35)*(1-isEyeInWater*0.8), reflection.rgb, reflection.a)*0.5;			//fake sky reflection, avoid empty spaces
		reflection.a = min(reflection.a + 1.0*sky_lightmap,0.5);
		color.rgb += reflection.rgb * mix(vec3(1.0/sqrt(3.0)),normalize(color.rgb),0.01) * (iswater)*fresnel * 3 * (1-TimeMidnight*0.4) * (1-rainx*0.75*TimeMidnight) * (1-TimeSunrise*0.4) * (1-TimeSunset*0.4) * REFLECTION_STRENGTH*reflection.a;
		color.rgb += color.a*pow(sunlight*0.5,vec3(1.0/7.0))*(1.0-TimeMidnight*0.75)*4.7;
		#endif
    }	
	
		if (isEyeInWater < 0.9 && iswater < 0.9) {
		reflection = raytraceGround(fragpos, normal);
		reflection.rgb = mix(skycolorreflection*2*(1.0-TimeMidnight*0.8)*(1.0-TimeSunrise*0.35)*(1.0-TimeSunset*0.35), reflection.rgb, reflection.a);			//fake sky reflection, avoid empty spaces
		reflection.a = min(reflection.a + 1.0*sky_lightmap,0.5);
		
	#ifdef RAIN_REFLECTIONS
		color.rgb += (reflection.rgb*fresnel*reflection.a)*iswet*(1.0-TimeMidnight*0.5)*(1.0-specmap)* getRainPuddles(fragpos);
		color.rgb += (reflection.rgb*fresnel*reflection.a)*.1*iswet*(1.0-TimeMidnight*0.5)*(1.0-specmap)* (1-getRainPuddles(fragpos));
		color.rgb += color.a*pow(sunlight,vec3(1.0/7.0))*iswet*(1.0-TimeMidnight)*4.7* getRainPuddles(fragpos);
	#endif
	#ifdef SPECULAR_REFLECTIONS
		color.rgb += (reflection.rgb*fresnel*specmap *reflection.a)*2;
	#endif
 }
	
	vec3 colmult = mix(vec3(1.0),vec3(0.08,0.237,0.47)/1.0,isEyeInWater);
	float depth_diff = clamp(pow(ld(texture2D(gdepthtex, texcoord.st).r)*4,2.0),0.0,0.8);
	color.rgb = mix(color.rgb,vec3(0.024,0.050,0.08)/4.0,depth_diff*(1.0-TimeMidnight*-0.4)*isEyeInWater);

		
		float time = float(worldTime);
		float transition_fading = 1.0-(clamp((time-12000.0)/300.0,0.0,1.0)-clamp((time-13500.0)/300.0,0.0,1.0) + clamp((time-22500.0)/300.0,0.0,1.0)-clamp((time-23400.0)/300.0,0.0,1.0));	//fading between sun/moon shadows
		
		vec3 scss = vec3(254,161,30)/254;
		vec3 scsr = vec3(2,1.5,1)/2;
		vec3 scn = vec3(1.25,1.5,2)/2;
		vec3 scnoon = vec3(189,209,244)/244;
		
		float horizon = (worldposition.y - (texcoord.y-cameraPosition.y));
		float calcHeight = (max(pow(max(2.0 - horizon/100.0, 0.0), 8.0)-0.1, 0.0));
		float calcHeight2 = (max(pow(max(1.9 - horizon/100.0, 0.0), 8.0)-0.1, 0.0));
		
		float volumetric_cone = pow(max(dot(normalize(fragpos),lightVector),0.0),2.5)*transition_fading;
		float volumetric_cone2 = pow(max(dot(normalize(fragpos),lightVector),0.0),10.0*200.0)*transition_fading;
		float volumetric_cone3 = pow(max(dot(normalize(fragpos),lightVector),0.0),10.0*25.0)*transition_fading;
		float volumetric_cone4 = pow(max(dot(normalize(fragpos),lightVector),0.0),10.0*5.0)*transition_fading;
		float volumetric_cone5 = pow(max(dot(normalize(fragpos),-lightVector),0.0),7.0)*transition_fading;
		
		float norain = 1.0-rainx;
		
		if (isEyeInWater < .9) {
		
			float fog = exp(-pow(length(fragpos)/far ,2.0-(1.0*TimeSunrise*0.3)+(1.5*rainx*1.0*TimeSunrise)));
			float fog2 = exp(-pow(length(fragpos)/20 ,2.0))*rainx;
			float fog3 = exp(-pow(length(fragpos)/30 ,2.0));
			float fogfactor =  clamp(fog + + hand + isEyeInWater,0.0,1.0);
			float fogfactor2 =  clamp(fog2 + + hand + isEyeInWater,0.0,1.0)*rainx;
			float fogfactor3 =  clamp(fog3 + + hand + isEyeInWater,0.0,1.0);

			fogclr = mix(color.rgb,ambient_color+.2*0.15*(1.0-TimeSunset*0.5),0.6)*(1.0-rainx)*(1.0-TimeMidnight*0.4);
			
			fogclr2 = mix(color.rgb,ambient_color*0.0,0.0);
			
			fogclr3 = mix(color.rgb,rainfogclr*8.0,1.0)*rainx;
			fogclr3 += mix(color.rgb,ambient_color,1.0)*rainx*TimeMidnight;
			fogclr3 -= fogclr3*0.8;
			
			vec3 limitcsssr = vec3(250,150,50)/250*(1.0-TimeNoon)*(1.0-TimeMidnight);
			
			
			//glow
			
			fogclr += vec3(1,1,1.5)/30 * volumetric_cone*1.0*(1.0-TimeMidnight*0)*(1.0-TimeNoon)*(1.0-TimeSunrise)*(1.0-TimeSunset)*(1.0-rainStrength)*1.0;
			fogclr2 += vec3(1,1,1.5)/30 * volumetric_cone*1.0*(1.0-TimeMidnight*-1)*(1.0-TimeNoon)*(1.0-TimeSunrise)*(1.0-TimeSunset)*(1.0-rainStrength)*1.0;
			
			fogclr += limitcsssr* volumetric_cone*0.5*(1.0-TimeMidnight)*(1.0-TimeNoon)*(1.0-rainStrength*0.4)*1.0;
			fogclr2 += limitcsssr* volumetric_cone*0.5*(1.0-TimeMidnight)*(1.0-TimeNoon)*(1.0-rainStrength*0.4)*1.0;
			
			fogclr += volumetric_cone*0.15*+(1.0-TimeMidnight)*(1.0-TimeSunrise)*(1.0-TimeSunset)*(1.0-rainStrength)*1.0;
			fogclr2 += volumetric_cone*0.15*+(1.0-TimeMidnight)*(1.0-TimeSunrise)*(1.0-TimeSunset)*(1.0-rainStrength)*1.0;
			
			//sunset glow at night	
			
			//sun
			if (land > 0.1) {
				fogclr2 += volumetric_cone2*100.0*(1.0-TimeMidnight)*(1.0-rainStrength)*(1.0-iswater*0.37);
				fogclr2 += sunlight*volumetric_cone4*(1.0-TimeMidnight)*(1.0-rainStrength)*(1.0-iswater*0.37);
			
			//moon
				fogclr2 += (moonlight*1+ambient_color*0.1+1*volumetric_cone3*0.3*(1.0-iswater*0.8))*TimeMidnight;
			}
			
			if (land < 0.1) {
				color.rgb = mix(fogclr*.35,color.rgb,fogfactor)*(1.0-rainx*(0.8*(1.0-TimeMidnight*0.5)));
				color.rgb += color.a*pow(sunlight,vec3(5.0/7.0));
				color.rgb += mix(fogclr3,color.rgb,fogfactor2)*rainx*0.775*(1.0-TimeMidnight);
			}
			
			if (land > 0.1) {
				color.rgb = mix(fogclr2,color.rgb,fogfactor)*(1.0-TimeMidnight*0.6)*(1.0-rainx*0.75);
				color.rgb += color.a*pow(sunlight,vec3(5.0/7.0));
				color.rgb += vec3(rainfogclr*rainStrength)*(1.0*rainStrength)*(1.0-TimeMidnight);
				color.rgb -= (color.rgb*0.3*TimeMidnight)*rainStrength*transition_fading;
				color.rgb += fogclr*TimeMidnight*0.1;
			}	
			
			color.rgb += mix(fogclr3,color.rgb,fogfactor2)*rainx;
			color.rgb -= (color.rgb*.5*(1-fogfactor2))*rainx*TimeMidnight*transition_fading;
			
			//altitude fog
			if (land < 0.1) {
				color.rgb += mix(fogclr*.5*(1-fogfactor)*(max(1-pow(fogclr, vec3(calcHeight)), 0.0)),color.rgb,fogfactor2)*(1-rainx);
			}
			
			//color.rgb += mix(vec3(1)*(1-fogfactor3)*(max(1-pow(vec3(calcHeight2), vec3(1)), 0.0)),color.rgb,fogfactor2)*(1-TimeMidnight*0.5)*(1-rainx);
			
			if (land > 0.1) {
				color.rgb -= color.rgb*0.07*rainx;
			}
		}
		
/* DRAWBUFFERS:5 */
		vec4 tpos = vec4(sunPosition,1.0)*gbufferProjection;
		tpos = vec4(tpos.xyz/tpos.w,1.0);
		vec2 pos1 = tpos.xy/tpos.z;
		vec2 lightPos = pos1*0.5+0.5;

	
	#ifdef GODRAYS
	
	const int nSteps = NUM_SAMPLES;
	const float blurScale = 0.002/nSteps*9.0;
	const int center = (nSteps-1)/2;
	float tw = 0.0;
	const float sigma = 0.5;
	
	float truepos = pow(clamp(dot(-lightVector,tpos.xyz)/length(tpos.xyz),0.0,1.0),0.25);
	
	vec3 rainc = mix(vec3(1.),fogclr3*1.5,rainStrength);
	vec3 lightColor = mix(sunlight*sunVisibility*rainc,3*moonlight*moonVisibility*rainc,(truepos+1.0)/2.);
	
	if (truepos > 0.05 && isEyeInWater < 0.9) {
	vec2 deltaTextCoord = normalize(texcoord.st - lightPos.xy)*blurScale*density;
    vec2 textCoord = texcoord.st;
    float illuminationDecay = 1.0;
	vec2 noise = vec2(getnoise(textCoord),getnoise(-textCoord.yx+0.05));
	float gr = 0.0;
	float avgdecay = 0.0;
	
			float distx = abs(texcoord.x*aspectRatio-lightPos.x*aspectRatio);
			float disty = abs(texcoord.y-lightPos.y);
            illuminationDecay = pow(max(1.0-sqrt(distx*distx+disty*disty),0.0),1.5*GODRAYS_SMOOTHNESS);
			
    for(int i=0; i < NUM_SAMPLES ; i++)
    {
		
           				textCoord += deltaTextCoord;
				
				float dist = (i-float(center))/center;
				float weight = exp(-(dist*dist)/(2.0*sigma));
				
				float sample = texture2D(gdepth, textCoord).r*weight;
				
				tw += weight;
				gr += sample;

    }
	color.rgb = mix(color.rgb,pow(sunlight+(rainc)*rainStrength,vec3(1.0/8.0)),(gr/tw)*exposure*length(pow(sunlight,vec3(1.0/2.2)))*(1-TimeNoon*0.5)*(1-TimeMidnight*0.5)*(1-rainStrength*0.75)*illuminationDecay*truepos/sqrt(3.0)*transition_fading);
	}
	#endif
	
float visiblesun = 0.0;
float temp;
int nb = 0;
			
//calculate sun occlusion (only on one pixel) 
if (texcoord.x < pw && texcoord.x < ph) {
	for (int i = 0; i < 6;i++) {
		for (int j = 0; j < 6 ;j++) {
		temp = texture2D(gaux1,lightPos + vec2(pw*(i-0.0)*2.0,ph*(j-0.0)*2.0)).g;
		visiblesun +=  1.0-float(temp > 0.04) ;
		nb += 1;
		}
	}
	visiblesun /= nb;

}

	gl_FragData[0] = vec4(color.rgb,visiblesun);
	
}
