#version 120

/*
Modified Chocapic13 Shader Lite By Airloocke42
*/

#define MAX_COLOR_RANGE 16.0

//-------Effects-------//

#define LENS_EFFECTS

#define DOF
	#define DOF_INTENSITY 2.5

#define BLOOM
	const float b_rad = 1.2;	
	const float b_intensity = 8.0;	

#define VIGNETTE
	#define VIGNETTE_STRENGTH 8.25
	#define VIGNETTE_START 0.25
	#define VIGNETTE_EXP 1.5   

//-------Effects-------//

varying vec4 texcoord;
varying vec3 sunlight;
varying vec3 ambient_color;

uniform sampler2D depthtex0;
uniform sampler2D depthtex1;
uniform sampler2D gaux1;
uniform sampler2D gaux2;
uniform sampler2D gaux4;
uniform sampler2D gnormal;
uniform sampler2D gdepth;
uniform sampler2D gcolor;

uniform sampler2D composite;

uniform vec3 cameraPosition;
uniform vec3 previousCameraPosition;
uniform vec3 sunPosition;
uniform vec3 moonPosition;

uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferPreviousProjection;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferPreviousModelView;

uniform ivec2 eyeBrightness;

uniform int isEyeInWater;
uniform int worldTime;

uniform float aspectRatio;
uniform float near;
uniform float far;
uniform float viewWidth;
uniform float viewHeight;
uniform float rainStrength;
uniform float wetness;
uniform float frameTimeCounter;

vec3 sunPos = sunPosition;

uniform int fogMode;

float timefract = worldTime;

float pw = 1.0/ viewWidth;
float ph = 1.0/ viewHeight;

float rainx = clamp(rainStrength, 0.0f, 1.0f)/1.0f;
float wetx  = clamp(wetness, 0.0f, 1.0f);

float sky_lightmap = texture2D(gaux1,texcoord.xy).r;

float TimeMidnight = ((clamp(timefract, 12000.0, 12750.0) - 12000.0) / 750.0) - ((clamp(timefract, 23000.0, 24000.0) - 23000.0) / 1000.0);

vec3 fogclr = pow(mix(vec3(0.5,0.5,1.0),vec3(0.3,0.3,0.3),rainStrength)*ambient_color,vec3(2.2));

vec3 fragpos = vec3(texcoord.st, texture2D(depthtex0, texcoord.st).r);
vec3 normal = texture2D(gnormal, texcoord.st).rgb * 2.0 - 1.0;

float matflag = texture2D(gaux1,texcoord.xy).g;

float ld(float depth) { return (2.0 * near) / (far + near - depth * (far - near)); }

float A = 0.15, B = 0.35, C = 0.10, D = 0.20, E = 0.02, F = 0.30, W = MAX_COLOR_RANGE;

vec3 Uncharted2Tonemap(vec3 x) { return ((x*(A*x+C*B)+D*E)/(x*(A*x+B)+D*F))-E/F; }

float luma(vec3 color) { return dot(color,vec3(0.299, 0.587, 0.114)); }

float getnoise(vec2 pos) { return abs(fract(sin(dot(pos ,vec2(18.9898f,28.633f))) * 4378.5453f)); }

float distratio(vec2 pos, vec2 pos2, float ratio) {
float xvect = pos.x*ratio-pos2.x*ratio;
float yvect = pos.y-pos2.y;
return sqrt(xvect*xvect + yvect*yvect);
}

float gen_circular_lens(vec2 center, float size) {
return 1.0-pow(min(distratio(texcoord.xy,center,aspectRatio),size)/size,3.0);
}

vec3 genlens(vec3 lenscolor, float dist, float size, vec4 tpos, float sun) {

	vec2 lightPos = tpos.xy/tpos.z*dist; lightPos = (lightPos + 1.0f)/2.0f;	

    float lensFlare = max(pow(max(1 - pow(min(distratio(lightPos.xy, texcoord.xy, aspectRatio),size)/size,10.0)/1,0.1),5.0)-0.1,0);

    return clamp(lensFlare, 0.0, 1.0) * lenscolor * 0.5 * sun * (1.0-rainStrength);	
}

const vec2 hex_offsets[60] = vec2[60] (	vec2(  0.2165,  0.1250 ),
											vec2(  0.0000,  0.2500 ),
											vec2( -0.2165,  0.1250 ),
											vec2( -0.2165, -0.1250 ),
											vec2( -0.0000, -0.2500 ),
											vec2(  0.2165, -0.1250 ),
											vec2(  0.4330,  0.2500 ),
											vec2(  0.0000,  0.5000 ),
											vec2( -0.4330,  0.2500 ),
											vec2( -0.4330, -0.2500 ),
											vec2( -0.0000, -0.5000 ),
											vec2(  0.4330, -0.2500 ),
											vec2(  0.6495,  0.3750 ),
											vec2(  0.0000,  0.7500 ),
											vec2( -0.6495,  0.3750 ),
											vec2( -0.6495, -0.3750 ),
											vec2( -0.0000, -0.7500 ),
											vec2(  0.6495, -0.3750 ),
											vec2(  0.8660,  0.5000 ),
											vec2(  0.0000,  1.0000 ),
											vec2( -0.8660,  0.5000 ),
											vec2( -0.8660, -0.5000 ),
											vec2( -0.0000, -1.0000 ),
											vec2(  0.8660, -0.5000 ),
											vec2(  0.2163,  0.3754 ),
											vec2( -0.2170,  0.3750 ),
											vec2( -0.4333, -0.0004 ),
											vec2( -0.2163, -0.3754 ),
											vec2(  0.2170, -0.3750 ),
											vec2(  0.4333,  0.0004 ),
											vec2(  0.4328,  0.5004 ),
											vec2( -0.2170,  0.6250 ),
											vec2( -0.6498,  0.1246 ),
											vec2( -0.4328, -0.5004 ),
											vec2(  0.2170, -0.6250 ),
											vec2(  0.6498, -0.1246 ),
											vec2(  0.6493,  0.6254 ),
											vec2( -0.2170,  0.8750 ),
											vec2( -0.8663,  0.2496 ),
											vec2( -0.6493, -0.6254 ),
											vec2(  0.2170, -0.8750 ),
											vec2(  0.8663, -0.2496 ),
											vec2(  0.2160,  0.6259 ),
											vec2( -0.4340,  0.5000 ),
											vec2( -0.6500, -0.1259 ),
											vec2( -0.2160, -0.6259 ),
											vec2(  0.4340, -0.5000 ),
											vec2(  0.6500,  0.1259 ),
											vec2(  0.4325,  0.7509 ),
											vec2( -0.4340,  0.7500 ),
											vec2( -0.8665, -0.0009 ),
											vec2( -0.4325, -0.7509 ),
											vec2(  0.4340, -0.7500 ),
											vec2(  0.8665,  0.0009 ),
											vec2(  0.2158,  0.8763 ),
											vec2( -0.6510,  0.6250 ),
											vec2( -0.8668, -0.2513 ),
											vec2( -0.2158, -0.8763 ),
											vec2(  0.6510, -0.6250 ),
											vec2(  0.8668,  0.2513 ));

void main() {

	int iswater = int(matflag > 0.04 && matflag < 0.07);
	int land = int(matflag > 0.04);
	int hand  = int(matflag > 0.75 && matflag < 0.85);

	vec2 fake_refract = vec2(sin(worldTime/15.0 + texcoord.x*100.0 + texcoord.y*50.0),cos(worldTime/15.0 + texcoord.y*100.0 + texcoord.x*50.0)) * isEyeInWater;

	vec3 color = texture2D(gaux2, texcoord.st + fake_refract * 0.005).rgb*MAX_COLOR_RANGE;

	float fog = 0.0, sxt = MAX_COLOR_RANGE; 
	fog = 1.0-clamp(exp(-pow(ld(texture2D(depthtex0, texcoord.st).r),3.0)),0.0,1.0);
	
	#ifdef DOF
	vec4 bcolor = vec4(0.0);
	
	float tex = texture2D(depthtex0, texcoord.st).x * far;
	float focus = texture2D(depthtex0, vec2(0.5)).x * far;
	float blur = pow((tex - focus) * 0.3, 1.0);		

	if (hand < 0.9) {

	for ( int i = 0; i < 60; i++) {

   	vec2 depth = hex_offsets[i]/512 * vec2(1.0,aspectRatio);
	depth *= blur;

    	bcolor += texture2D(gaux2, texcoord.xy + depth)*sxt;
	}
   	color.rgb = bcolor.rgb/61.0;
	}
	#endif

	#ifdef BLOOM
	vec3 csample = vec3(0.0); float scale = length(vec2(pw,ph));

	for (int i = -3; i < 3; i++) {
		for (int j = -3; j < 3; j++) {

	    vec2 coords = (vec2(j, i)/2.5*1.5), aspr = vec2(1.0,aspectRatio);
	    vec2 bloom_sfx = (texcoord.xy + coords*b_rad*scale*aspr);

	    float weight = exp(-length(vec2(j, i)/2.5));

	   csample += pow(texture2D(gaux2, bloom_sfx).rgb*weight,vec3(2.2))*32.0;
		}
	}		
	color += max(csample/37.5*b_intensity,0.0);
	#endif

	float time = float(worldTime);
	float transition_fading = 1.0-(clamp((time-12000.0)/300.0,0.0,1.0)-clamp((time-13000.0)/300.0,0.0,1.0) + clamp((time-22800.0)/200.0,0.0,1.0)-clamp((time-23400.0)/200.0,0.0,1.0));

	fogclr = mix(color.rgb,ambient_color*0.5,0.2);

	color.rgb += texture2D(gaux4,texcoord.xy).rgb*sqrt(texture2D(gaux4,texcoord.xy).a);

	vec4 tpos = vec4(sunPosition,1.0)*gbufferProjection;
		tpos = vec4(tpos.xyz/tpos.w,1.0);
		vec2 pos1 = tpos.xy/tpos.z;
		vec2 lightPos = pos1*0.5+0.5;
		
		
		vec3 lightVector;
	if (worldTime < 12700 || worldTime > 23250) {
		lightVector = normalize(sunPosition);
	} else {
		lightVector = normalize(moonPosition);
	}

	float xdist = abs(lightPos.x-texcoord.x);
	float ydist = abs(lightPos.y-texcoord.y);

	float distof = min(min(1.0-lightPos.x,lightPos.x),min(1.0-lightPos.y,lightPos.y));
	float fading = clamp(1.0-step(distof,0.1)+pow(distof*10.0,5.0),0.0,1.0);

	float sunvisibility = min(texture2D(gaux2,vec2(0.0)).a*2.5,1.0) * (1.0-rainStrength*0.9) * fading * transition_fading;

	float truepos = pow(clamp(dot(-lightVector,tpos.xyz)/length(tpos.xyz),0.0,1.0),0.25);

	#ifdef LENS_EFFECTS
	if (sunvisibility > 0.05) {

	float anamorphic_lens = exp(-ydist*ydist/0.005)*exp(-xdist*xdist/0.1)*sunvisibility*(1-TimeMidnight*0.9) * (1-rainStrength);

	color.rgb += (vec3(0.25,0.45,1.0)*anamorphic_lens);
	}

	vec3 light_color = mix(vec3(1.0),sunlight,sunvisibility);

	float sun = sunvisibility;
	float h = TimeMidnight, r = rainStrength;

	if (sun > 0.05) {

	vec3 c2 = vec3(0.1,0.6,.8) * (1-h), c1 = vec3(0.8,0.6,.1) * (1-h);
	vec3 c0 = vec3(0.1,1.0,.3) * (1-h), c3 = vec3(1.0,0.3,.1) * (1-h);

	color += genlens(c3, -0.05, 0.05, tpos, sun * (1-r));
	color += genlens(c1, -0.25, 0.07, tpos, sun * (1-r));
	color += genlens(c2, -0.45, 0.09, tpos, sun * (1-r));
	color += genlens(c0, -0.65, 0.13, tpos, sun * (1-r));

	}

	#endif

	#ifdef VIGNETTE

	float dv = pow(clamp(distance(texcoord.st, vec2(0.5, 0.5))/sqrt(2.0)-VIGNETTE_START,0.0,1.0),VIGNETTE_EXP);

	dv *= VIGNETTE_STRENGTH;
	dv = clamp(1.0 - dv,0.0,1.0);

	color *= dv;
	#endif

	float avglight = texture2D(gaux2,vec2(1.0)).a;
	float exposure = -log(clamp(avglight*48.0,0.01,0.5));
	exposure = 1.0;

	vec3 curr = Uncharted2Tonemap(color);

	vec3 whiteScale = 1.0f/Uncharted2Tonemap(vec3(W));
	color = curr*whiteScale;

	float saturation = 1.05, avg = (color.r + color.g + color.b);

	/*
	float white = luma(color);
	color = color*(1.0+pow(white,0.2))/(2.0-0.2);
	color = color / (color + 1.0) * (1.0+1.0);
	*/

        color = (((color - avg )*saturation)+avg) ;
	color /= saturation;	
	color = clamp(pow(color,vec3(1.0/2.2)),0.0,1.0);

	gl_FragColor = vec4(color,1.0);
	
}